################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################



import sys, os
import numpy as npy
import modellib
from natconst import *
class GridCart2D():
    def __init__(self,xmin=None,xmax=None,nx=None,xl=None,xpos=None,xval=0.0,
                 ymin=None,ymax=None,ny=None,yl=None,ypos=None,yval=0.0,
                 zmin=None,zmax=None,nz=None,zl=None,zpos=None,zval=0.0):

        

        xmin = 0.0
        ymin = 0.0
        zmin = 0.0

        if (ymin==None):
            ymin = xmin
            ymax = xmax
            ny   = nx
        if (zmin==None):
            zmin = xmin
            zmax = xmax
            nz   = nx
        if (yl==None):
            yl = xl
        if (zl==None):
            zl = xl

        self.nx = nx
        self.ny = ny
        self.nz = nz

        
        if (nx==1):
            self.x = npy.arange(nx)*0.0 + xval
        else:
            if (xl==None):
                if (xpos!=None):
                    self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax
                else:
                    self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax
            else:
                self.x = 10.**(npy.arange(nx) / (nx-1.) * (npy.log10(xmax) - npy.log10(xmin)) +\
                                      npy.log10(xmin))

        if (ny==1):
            self.y = npy.arange(ny)*0.0 + yval
        else:
            if (yl==None):
                if (ypos!=None):
                    self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
                else:
                    self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
            else:
                self.y = 10.**(npy.arange(ny) / (ny-1.) * (npy.log10(ymax) - npy.log10(ymin)) +\
                                  npy.log10(ymin))

            
        if (nz==1):
            self.z = npy.arange(nz)*0.0 + zval
        else:
            if (zl==None):
                if (zpos!=None):
                    self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
                else:
                    self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
            else:
                self.z = 10.**(npy.arange(nz) / (nz-1.) * (npy.log10(zmax) - npy.log10(zmin)) +\
                                  npy.log10(zmin))  
class GridCart():
    def __init__(self,xmin=None,xmax=None,nx=None,xl=None,xpos=None,
                 ymin=None,ymax=None,ny=None,yl=None,ypos=None,
                 zmin=None,zmax=None,nz=None,zl=None,zpos=None):

        
        xmin = 0.0
        ymin = 0.0
        zmin = 0.0

        if (ymin==None):
            ymin = xmin
            ymax = xmax
            ny   = nx
        if (zmin==None):
            zmin = xmin
            zmax = xmax
            nz   = nx
        if (yl==None):
            yl = xl
        if (zl==None):
            zl = xl

        self.nx = nx
        self.ny = ny
        self.nz = nz

        
        if (xl==None):
            if (xpos!=None):
                self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax
            else:
                self.x = (npy.arange(nx) / (nx-1.) - 0.5) * 2.0 * xmax
        else:
            self.x = 10.**(npy.arange(nx) / (nx-1.) * (npy.log10(xmax) - npy.log10(xmin)) +\
                                  npy.log10(xmin))

        if (yl==None):
            if (ypos!=None):
                self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
            else:
                self.y = (npy.arange(ny) / (ny-1.) - 0.5) * 2.0 * ymax
        else:
            self.y = 10.**(npy.arange(ny) / (ny-1.) * (npy.log10(ymax) - npy.log10(ymin)) +\
                                  npy.log10(ymin))

        if (zl==None):
            if (zpos!=None):
                self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
            else:
                self.z = (npy.arange(nz) / (nz-1.) - 0.5) * 2.0 * zmax
        else:
            self.z = 10.**(npy.arange(nz) / (nz-1.) * (npy.log10(zmax) - npy.log10(zmin)) +\
                                  npy.log10(zmin))  
class GridSph():
    def __init__(self,rmin=None,rmax=None,nr=None,rl=None,
                 tmin=None,tmax=None,nt=None,
                 pmin=None,pmax=None,np=None):
        
        self.nr  = nr
        self.nt  = nt
        self.np  = np
        if (rl==True):
            self.r    = 10.**(npy.arange(nr*1.0) / (nr-1.) * (npy.log10(rmax) - npy.log10(rmin)) +\
                                  npy.log10(rmin))
        else:
            self.r    = npy.arange(nr*1.0) / (nr-1.) * (rmax-rmin) + rmin

        self.theta  = npy.arange(nt*1.0) / (nt-1.) * (tmax-tmin) + tmin
        self.phi  = npy.arange(np*1.0) / (np-1.) * (pmax-pmin) + pmin
    
class getPlotModelVar():

    def __init__(self, GridPar, model_id=0,  varname=' ', defaultPar=False, UnitSystem='SI'):
        nc      = natconst()


        grid_type = 0        
        if (grid_type==0):
            grid = GridCart2D(xmin=GridPar.xmin,xmax=GridPar.xmax,nx=GridPar.nx,xpos=GridPar.xpos,xval=GridPar.xval,
                                    ymin=GridPar.ymin,ymax=GridPar.ymax,ny=GridPar.ny,ypos=GridPar.ypos,yval=GridPar.yval,
                                    zmin=GridPar.zmin,zmax=GridPar.zmax,nz=GridPar.nz,zpos=GridPar.zpos,zval=GridPar.zval)
            self.var = npy.zeros((grid.nx,grid.ny,grid.nz),dtype=npy.float64)        
            self.x = grid.x
            self.y = grid.y
            self.z = grid.z
            self.nx = GridPar.nx
            self.ny = GridPar.ny
            self.nz = GridPar.nz
        else:
            print 'Not supported grid style'
            sys.exit()

        if UnitSystem=='CGS':
            unitconv = {'density':1e-6, 'temperature':1.0, 'velocity':1e2}
        else:
            unitconv = {'density':1.0, 'temperature':1.0, 'velocity':1.0}


                
        if (grid_type==0):
            if (varname=='density'):
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        for iz in range(grid.nz):
                            self.var[ix][iy][iz]  = modellib.density(grid.x[ix],grid.y[iy],grid.z[iz]) * unitconv['density']
            elif (varname=='temperature'):
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        for iz in range(grid.nz):
                            self.var[ix][iy][iz]  = modellib.temperature(grid.x[ix],grid.y[iy],grid.z[iz]) * unitconv['temperature']
            elif (varname=='velocity_x'):
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        for iz in range(grid.nz):
                            dum  = modellib.velocity(grid.x[ix],grid.y[iy],grid.z[iz])
                            self.var[ix][iy][iz] = dum[0] * unitconv['velocity']
            elif (varname=='velocity_y'):
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        for iz in range(grid.nz):
                            dum  = modellib.velocity(grid.x[ix],grid.y[iy],grid.z[iz]) 
                            self.var[ix][iy][iz] = dum[1] * unitconv['velocity']
            elif (varname=='velocity_z'):
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        for iz in range(grid.nz):
                            dum  = modellib.velocity(grid.x[ix],grid.y[iy],grid.z[iz])
                            self.var[ix][iy][iz] = dum[2] * unitconv['velocity']
            elif (varname=='abundance'):
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        for iz in range(grid.nz):
                            self.var[ix][iy][iz] = modellib.abundance(grid.x[ix],grid.y[iy],grid.z[iz])

            else:
                for ix in range(grid.nx):
                    for iy in range(grid.ny):
                        for iz in range(grid.nz):
                            self.var[ix][iy][iz] = 0.0
  
def setDefaultParamValues(ModelID):
        
    paramIDs = modellib.getParamIDs(ModelID)

    for i in range(len(paramIDs)):
        paramType = modellib.getParamType(ModelID, paramIDs[i])
        
        if paramType=='int':
            modellib.setParamInt(paramIDs[i], modellib.getParamDefValInt(ModelID, paramIDs[i]))

        if paramType=='double':
            modellib.setParamDouble(paramIDs[i], modellib.getParamDefValDouble(ModelID, paramIDs[i]))

        if paramType=='enum':
            modellib.setParamEnumIndex(paramIDs[i], modellib.getParamEnumDefIndex(ModelID, paramIDs[i]))

        if paramType=='string':
            if paramIDs[i]=='dustopac_fname':
                modellib.setParamString(paramIDs[i], os.environ['ARTIST_ROOT']+'/example/jena_thin_e6.tab')
            else:
                modellib.setParamString(paramIDs[i], modellib.getParamDefValString(ModelID, paramIDs[i]))

    return


